@@ -1,7 +1,7 @@
 "print.permutest.cca" <-
     function (x, ...) 
 {
-    cat("\nPermutation test for a CCA/RDA call:\n")
+    cat("\nPermutation test for", x$method, "\nCall:\n")
     cat(deparse(x$call), "\n\n")
     Pval <- sum(x$F.perm >= x$F.0)/x$nperm
     cat("Test for significance of all constrained eigenvalues\n")
